% set the matrix A, the intital condition Y(0) and step size h

A = [0, 1; -1, 0];
Y0 = [1, 1; -1, 1] / sqrt(2);
h = 1/20;

% starting values for the iteration
Yexpleul = Y0;
Yimpleul = Y0;
Yimplmidp = Y0;

% allocate memory for results (||Y_k'Y_k-I||_F)
normexpleul = zeros(20, 1);
normimpleul = zeros(20, 1);
normimplmidp = zeros(20, 1);

for k = 1:20
	
    % compute the steps using the explicit, implicit Euler and implicit
    % midpoint rule
	Yexpleul = ExplEulStep(A, Yexpleul, h);
	Yimpleul = ImplEulStep(A, Yimpleul, h);
	Yimplmidp = ImplMidpStep(A, Yimplmidp, h);
	
    % compute the norms
	normexpleul(k) = norm(Yexpleul'*Yexpleul - eye(2), 'fro');
	normimpleul(k) = norm(Yimpleul'*Yimpleul - eye(2), 'fro');
	normimplmidp(k) = norm(Yimplmidp'*Yimplmidp - eye(2), 'fro');
end


% plot the results

fprintf('Explicit Euler: %.2e\n', normexpleul(20));
fprintf('Implicit Euler: %.2e\n', normimpleul(20));
fprintf('Implicit mid-point: %.2e\n', normimplmidp(20));

figure
semilogy(1:20, normexpleul, 'bo-', 1:20, normimpleul, 'g.-', 1:20, normimplmidp, 'r^-')
xlabel('Iterations', 'FontSize', 14)
ylabel('$\Vert Y_{k}^\top Y_{k}-I\Vert_F$', 'Interpreter', 'latex', 'FontSize', 14)
legend('explicit Euler', 'implicit Euler', 'implicit mid-point', 'Location', 'East')
